/* Breakpoint interface */
#ifndef BREAKPOINT_H__
#define BREAKPOINT_H__

#include <config.h>

#include <stdint.h>

#include <lib.h>

BEGIN_C_DECLS

/* A breakpoint */
struct breakpoint
{
  struct breakpoint *next;
  struct expr_code *compiled;
  intptr_t data; /* For purposes of the frontend. */
  char *expr;
};

/* Breakpoint list. Should be modified only by the breakpoint_* functions */
extern struct breakpoint *breakpoint_list;

/* Create a new breakpoint with EXPR;
   returns: new breakpoint or NULL on error */
extern struct breakpoint *
breakpoint_new (const char *expr);

/* Change BP to EXPR;
   returns: 0 if OK, -1 on error */
extern int
breakpoint_change (struct breakpoint *bp, const char *expr);

/* Delete BP */
extern void
breakpoint_delete (struct breakpoint *bp);

/* Check whether BP triggered */
extern int
breakpoint_eval (struct breakpoint *bp);

/* Check whether some breakpoint triggered;
   returns: first matching breakpoint or NULL if none */
extern struct breakpoint *
breakpoint_check (void);

/* Delete all known breakpoints */
extern void
breakpoint_delete_all (void);

END_C_DECLS

#endif
