/* Generic support library */
#ifndef LIB_H__
#define LIB_H__

#include <config.h>

#include <stdint.h>
#include <stdio.h>

#ifdef __cplusplus
#define BEGIN_C_DECLS extern "C" {
#define END_C_DECLS }
#else
#define BEGIN_C_DECLS /* empty */
#define END_C_DECLS /* empty */
#endif

BEGIN_C_DECLS

/* Provide portable form of GCC __attribute__ */
#ifdef __GNUC__
#define attribute__(foo) __attribute__ (foo)
#else
#define attribute__(foo)
#endif

/* Number of entries of ARRAY */
#define ARRAY_SIZE(array) (sizeof (array) / sizeof (*array))

struct _xmlDoc;
struct _xmlNode;
struct _xmlNs;

/* Program name for error reporting */
extern char *program_name;

/* Report printf-like ERR, append message for ERR_NO if ERR_NO != 0;
   Each frontend defines its own implementation. */
extern void attribute__ ((format (printf, 2, 3)))
error (int err_no, const char *err, ...);

/* Parse a number-only argument STRING with strtoul-like BASE, store result
   to *RES;
   returns: 0 if OK, -1 on error */
extern int
get_num (const char *string, int base, unsigned long *res);

/* Parse unsigned number from xml NODE of DOC in range [0, limit), store
   result to *RES;
   returns: 0 if OK, -1 on error */
extern int
get_xml_num (struct _xmlDoc *doc, struct _xmlNode *node, unsigned long limit,
	     unsigned long *res);

/* Load *VAL from xml element NODE of DOC, containing one of NUM STRINGS;
   returns: 0 if OK, -1 on error */
extern int
get_xml_enum (struct _xmlDoc *doc, struct _xmlNode *node,
	      const char *const *strings, size_t num, unsigned *val);

/* malloc () SIZE bytes of memory, error () on failure;
   returns: pointer to allocated area */
extern void *
xmalloc (size_t size);

/* realloc () PTR to SIZE bytes, error () on failure;
   returns: pointer to reallocated area */
extern void *
xrealloc (void *ptr, size_t size);

/* malloc () and make a copy of STR
   returns: pointer to string copy */
extern char *
xstrdup (const char *str);

/* xmlMalloc () SIZE bytes of memory, error () on failure;
   returns: pointer to allocated area */
extern void *
xxmlMalloc (size_t size);

/* Load Intel hex file FILENAME to DATA of LEN;
   returns: 0 if OK, -1 on error */
extern int
load_hex_file (const char *filename, uint8_t *data, size_t len);

/* Load Intel hex FILE (of NAME) to DATA of LEN;
   returns: 0 if OK, -1 on error */
extern int
load_hex_stdio (const char *name, FILE *file, uint8_t *data, size_t len);

/* Load Intel hex from XML NODE of DOC to DATA of LEN;
   returns: 0 if OK, -1 on error */
extern int
load_hex_xml (struct _xmlDoc *doc, struct _xmlNode *node, uint8_t *data,
	      size_t len);

/* Save DATA of LEN (at most 64K) as Intel hex to XML element NS:NAME under
   NODE */
extern void
save_hex_xml (struct _xmlNode *node, struct _xmlNs *ns, const char *name,
	      const uint8_t *data, size_t len);

END_C_DECLS

#endif
