/* "Setup" ("hardware configuration", modules, their instances and options)
   handling */
#ifndef SETUP_H__
#define SETUP_H__

#include <config.h>

#include <stdint.h>

#include <lib.h>

BEGIN_C_DECLS

struct _xmlNode;

/* Create an "inital" setup */
extern void
setup_init (void);

/* Delete current setup */
extern void
setup_destroy (void);

/* Save current setup to FILE;
   returns: 0 if OK, -1 on error */
extern int
setup_save (const char *file);

/* Load current setup from FILE;
   returns: 0 if OK, -1 on error (current setup is already destroyed) */
extern int
setup_load (const char *file);

/* Save current state to FILE, UI state is saved by FN with DATA;
   returns: 0 if OK, -=1 on error */
extern int
state_save (const char *file,
	    void (*fn) (struct _xmlNode *node, intptr_t data), intptr_t data);

/* Load current state from FILE (or create empty state if FILE is
   cereal_setup document), UI state is loaded by FN with DATA;
   returns: 1 if  loaded cereal_state, 0 if loaded cereal_setup, -1 on error
   (current state is already destroyed) */
extern int
state_load (const char *file, int (*fn) (struct _xmlDoc *doc,
					 struct _xmlNode *node, intptr_t data),
	    intptr_t data);

END_C_DECLS

#endif
