/* 8051 cereal_kde plugin */

#include <kaction.h>
#include <kfiledialog.h>
#include <klocale.h>
#include <libxml/tree.h>

#include <me.h>
#include <module.h>
#include "../kde/kcmainwin.h"

#include "kc8051.h"
#include "kc8051viewwin.h"

/* Our XML namespace */
extern const char KC8051XMLNs[] = "http://www.fw.cz/mitr/cereal/modules/8051";

KC8051::KC8051 (QObject *parent, const char *name)
  : KParts::Plugin (parent, name)
{
  i8051_module = cereal_module_find ("8051");
  if (i8051_module == NULL)
    return; /* No way. */

  if (!parent->inherits ("KCMainWin"))
    return; /* No way either. */
  mainWin = static_cast <KCMainWin *> (parent);

  new KAction (i18n ("New &View"), 0, this, SLOT (newView ()),
	       actionCollection (), "newview");
  new KAction (i18n ("&Load Program..."), 0, this, SLOT (loadProgram ()),
	       actionCollection (), "loadprogram");

  mainWin->registerStateHandler (this);
}

KC8051::~KC8051 ()
{
}

/** Save current UI state under uiState */
void
KC8051::saveState (xmlNodePtr)
{ /* Nothing */
}

/** Load UI state from uiState of doc;
    returns: 0 if OK, -1 on error */
int
KC8051::loadState (xmlDocPtr doc, xmlNodePtr uiState)
{
  int res = 0;
  for (xmlNodePtr node = uiState->children; node != NULL; node = node->next)
    {
      if (node->type != XML_ELEMENT_NODE
	  || strcmp ((const char *)node->name, "ui_window") != 0)
	continue;
      xmlChar *str = xmlGetProp (node, (const xmlChar *)"class");
      if (str == NULL)
	continue;
      if (strcmp ((const char *)str, "KC8051ViewWin") == 0)
	{
	  xmlChar *name = xmlGetNsProp (node, (const xmlChar *)"me",
					(const xmlChar *)KC8051XMLNs);
	  if (name == NULL)
	    {
	      res = -1;
	      goto err;
	    }
	  struct me_entry *me = me_get ((const char *)name);
	  xmlFree (name);
	  if (me == NULL)
	    {
	      res = -1;
	      goto err;
	    }
	  KC8051ViewWin *w = new KC8051ViewWin (mainWin, me,
						mainWin->viewParent ());
	  res = w->loadState (doc, node);
	  if (res == 0)
	    w->show ();
	  else
	    delete w;
	}
    err:
      xmlFree (str);
      if (res != 0)
	{
	  error (0, "Invalid 8051 ui_window state");
	  break;
	}
    }
  return res;
}

/** 8051/New View was selected */
void
KC8051::newView ()
{
  struct me_entry *me = mainWin->selectModule (i8051_module);
  if (me == NULL)
    return;
  /* Maybe we should not create duplicate windows */
  KC8051ViewWin *w = new KC8051ViewWin (mainWin, me, mainWin->viewParent ());
  w->show ();
}

/** 8051/Load Program was selected */
void
KC8051::loadProgram ()
{
  struct me_entry *me = mainWin->selectModule (i8051_module);
  if (me == NULL)
    return;
  QString fn = KFileDialog::getOpenFileName (0, "*.hex", mainWin);
  if (!fn.isEmpty ())
    me->module->set_option (me->mi, "load_hex", fn);
}
