/* Item/Insert dialog */

#include <klineedit.h>
#include <klistview.h>
#include <kpushbutton.h>
#include <kstdguiitem.h>
#include <qstring.h>
#include <libxml/tree.h>

#include <me.h>
#include <module.h>

#include "khwinsertdialog.h"

KHwInsertDialog::KHwInsertDialog (QWidget *parent)
  : KHwInsertDialogBase (parent)
{
  cancelButton->setGuiItem (KStdGuiItem::cancel ());
  OKButton->setGuiItem (KStdGuiItem::ok ());
  OKButton->setEnabled (false);

  for (const struct cereal_module *m = cereal_module_list; m != NULL;
       m = m->next)
    {
      QString descr("");
      if (m->xml != NULL)
	{
	  
	  xmlNodePtr node = xmlDocGetRootElement (m->xml);
	  xmlChar *val = xmlGetProp (node, (const xmlChar *)"description");
	  if (val != NULL)
	    {
	      descr = QString::fromUtf8 ((const char *)val);
	      xmlFree (val);
	    }
	}
      new KListViewItem (moduleType, m->name, descr);
    }
  moduleType->sort ();
  QListViewItem *first = moduleType->firstChild ();
  if (first != 0)
    {
      moduleType->setCurrentItem (first);
      moduleType->setSelected (first, false);
    }
}

KHwInsertDialog::~KHwInsertDialog ()
{
}

/** Get selected module type */
QString
KHwInsertDialog::getModuleType () const
{
  return moduleType->selectedItem ()->text (0);
}

/** Get selected module name */
QString
KHwInsertDialog::getModuleName () const
{
  return moduleName->text ();
}

/** Module name or module type selection has changed */
void
KHwInsertDialog::nameOrTypeChanged ()
{
  OKButton->setEnabled (moduleType->selectedItem () != 0
			&& !moduleName->text ().isEmpty ());
}

/** Check if moduleName is OK, and accept () if so */
void
KHwInsertDialog::tryAccept ()
{
  if (me_name_ok (moduleName->text ()))
    accept ();
}
