/* A simple module providing a write-only byte which discards the data  */
#include <config.h>

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libxml/tree.h>

#include <lib.h>
#include <module.h>

/* One bit-report instance state */
struct mi
{
  char pad;
};

/* Create a new module instance;
   returns: module instance pointer if OK, NULL on error */
static void *
byte_sink_mi_new (void)
{
  struct mi *mi;

  mi = xmalloc (sizeof (*mi));
  return mi;
}

/* Delete MI_ (may be in any emulation state) */
static void
byte_sink_mi_delete (void *mi_)
{
  struct mi *mi;

  mi = mi_;
  free (mi);
}

/* Save state of MI_ to NODE */
static void
byte_sink_save_state (void *mi_, xmlNodePtr node)
{
  (void)mi_;
  (void)node;
}

/* Load state of MI_ from NODE of DOC;
   returns: 0 if OK, -1 on error */
static int
byte_sink_load_state (void *mi_, xmlDocPtr doc, xmlNodePtr node)
{
  (void)mi_;
  (void)doc;
  (void)node;
  return 0;
}

/* Get number of ports of this space in MI_ */
static size_t
space_get_size (void *mi_)
{
  (void)mi_;
  return 1;
}

/* Write VAL to bit port in MI_ */
static void
space_write_fn (void *mi_, intptr_t data, uint8_t val)
{
  (void)mi_;
  (void)data;
  (void)val;
}

/* Save port_fn for PORT writing to DEST */
static void
space_get_write_fn (void *mi_, size_t port, struct port_fn *dest)
{
  assert (port == 0);
  dest->fn.write_8 = space_write_fn;
  dest->mi = mi_;
}

static const struct space byte_sink_space =
{
  .get_size = space_get_size, .get_fn =
  {
    [PORT_TYPE_READ] = NULL, [PORT_TYPE_WRITE] = space_get_write_fn,
    [PORT_TYPE_DISPLAY] = NULL, [PORT_TYPE_MODIFY] = NULL
  },
  .set_fn =
  {
    [PORT_TYPE_READ] = NULL, [PORT_TYPE_WRITE] = NULL,
    [PORT_TYPE_DISPLAY] = NULL, [PORT_TYPE_MODIFY] = NULL
  }
};

struct cereal_module byte_sink_module =
{
  .next = NULL, .pprev = NULL, .name = "byte_sink", .xml = NULL,
  .handle = NULL, .mi_new = byte_sink_mi_new,
  .mi_delete = byte_sink_mi_delete, .set_option = generic_module_set_option,
  .get_option = generic_module_get_option,
  .save_setup = generic_module_save_setup,
  .load_setup = generic_module_load_setup, .save_state = byte_sink_save_state,
  .load_state = byte_sink_load_state, .w =
  {
    [WIDTH_1] = { .spaces = NULL, .num_spaces = 0, .xml_spaces = NULL },
    [WIDTH_8] =
    { .spaces = &byte_sink_space, .num_spaces = 1, .xml_spaces = NULL },
    [WIDTH_16] = { .spaces = NULL, .num_spaces = 0, .xml_spaces = NULL }
  }
};
